@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import logging
 import ssl
@@ -11,9 +13,15 @@ from typing import Any, Callable
 from typing import Counter as typing_Counter
 from typing import Dict, List, Optional, Set, Tuple, Union
 
-from aiohttp import ClientSession, ClientTimeout, ServerDisconnectedError, WSCloseCode, client_exceptions, web
-from aiohttp.web_app import Application
-from aiohttp.web_runner import TCPSite
+from aiohttp import (
+    ClientResponseError,
+    ClientSession,
+    ClientTimeout,
+    ServerDisconnectedError,
+    WSCloseCode,
+    client_exceptions,
+    web,
+)
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes, serialization
@@ -28,9 +36,10 @@ from chia.server.ssl_context import private_ssl_paths, public_ssl_paths
 from chia.server.ws_connection import WSChiaConnection
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.peer_info import PeerInfo
+from chia.util.api_decorators import get_metadata
 from chia.util.errors import Err, ProtocolError
 from chia.util.ints import uint16
-from chia.util.network import is_in_network, is_localhost, select_port
+from chia.util.network import WebServer, is_in_network, is_localhost
 from chia.util.ssl_check import verify_ssl_certs_and_keys
 
 max_message_size = 50 * 1024 * 1024  # 50MB
@@ -99,6 +108,13 @@ def ssl_context_for_client(
     return ssl_context
 
 
+def calculate_node_id(cert_path: Path) -> bytes32:
+    pem_cert = x509.load_pem_x509_certificate(cert_path.read_bytes(), default_backend())
+    der_cert_bytes = pem_cert.public_bytes(encoding=serialization.Encoding.DER)
+    der_cert = x509.load_der_x509_certificate(der_cert_bytes, default_backend())
+    return bytes32(der_cert.fingerprint(hashes.SHA256()))
+
+
 class ChiaServer:
     def __init__(
         self,
@@ -119,17 +135,6 @@ class ChiaServer:
     ):
         # Keeps track of all connections to and from this node.
         self.all_connections: Dict[bytes32, WSChiaConnection] = {}
-        self.tasks: Set[asyncio.Task] = set()
-
-        self.connection_by_type: Dict[NodeType, Dict[bytes32, WSChiaConnection]] = {
-            NodeType.FULL_NODE: {},
-            NodeType.DATA_LAYER: {},
-            NodeType.WALLET: {},
-            NodeType.HARVESTER: {},
-            NodeType.FARMER: {},
-            NodeType.TIMELORD: {},
-            NodeType.INTRODUCER: {},
-        }
 
         self._port = port  # TCP port to identify our node
         self._local_type: NodeType = local_type
@@ -139,9 +144,6 @@ class ChiaServer:
         self._inbound_rate_limit_percent = inbound_rate_limit_percent
         self._outbound_rate_limit_percent = outbound_rate_limit_percent
 
-        # Task list to keep references to tasks, so they don't get GCd
-        self._tasks: List[asyncio.Task] = []
-
         self.log = logging.getLogger(name if name else __name__)
         self.log.info("Service capabilities: %s", self._local_capabilities_for_handshake)
 
@@ -157,25 +159,53 @@ class ChiaServer:
         if self._local_type is NodeType.INTRODUCER:
             self.introducer_peers = IntroducerPeers()
 
-        if self._local_type is not NodeType.INTRODUCER:
-            self._private_cert_path, self._private_key_path = private_ssl_paths(root_path, config)
-        if self._local_type is not NodeType.HARVESTER:
-            self.p2p_crt_path, self.p2p_key_path = public_ssl_paths(root_path, config)
+        ca_private_crt_path, ca_private_key_path = private_ca_crt_key
+        chia_ca_crt_path, chia_ca_key_path = chia_ca_crt_key
+
+        private_cert_path, private_key_path = None, None
+        public_cert_path, public_key_path = None, None
+
+        authenticated_client_types = {NodeType.HARVESTER}
+        authenticated_server_types = {NodeType.HARVESTER, NodeType.FARMER, NodeType.WALLET, NodeType.DATA_LAYER}
+
+        if self._local_type in authenticated_client_types:
+            # Authenticated clients
+            private_cert_path, private_key_path = private_ssl_paths(root_path, config)
+            self.ssl_client_context = ssl_context_for_client(
+                ca_private_crt_path, ca_private_key_path, private_cert_path, private_key_path
+            )
         else:
-            self.p2p_crt_path, self.p2p_key_path = None, None
-        self.ca_private_crt_path, self.ca_private_key_path = private_ca_crt_key
-        self.chia_ca_crt_path, self.chia_ca_key_path = chia_ca_crt_key
-        self.node_id = self.my_id()
+            # Public clients
+            public_cert_path, public_key_path = public_ssl_paths(root_path, config)
+            self.ssl_client_context = ssl_context_for_client(
+                chia_ca_crt_path, chia_ca_key_path, public_cert_path, public_key_path
+            )
+
+        if self._local_type in authenticated_server_types:
+            # Authenticated servers
+            private_cert_path, private_key_path = private_ssl_paths(root_path, config)
+            self.ssl_context = ssl_context_for_server(
+                ca_private_crt_path,
+                ca_private_key_path,
+                private_cert_path,
+                private_key_path,
+                log=self.log,
+            )
+        else:
+            # Public servers
+            public_cert_path, public_key_path = public_ssl_paths(root_path, config)
+            self.ssl_context = ssl_context_for_server(
+                chia_ca_crt_path, chia_ca_key_path, public_cert_path, public_key_path, log=self.log
+            )
+
+        # If node has public cert use that one for id, if not use private.
+        self.node_id = calculate_node_id(private_cert_path if public_cert_path is None else public_cert_path)
 
         self.incoming_task: Optional[asyncio.Task] = None
         self.gc_task: Optional[asyncio.Task] = None
-        self.app: Optional[Application] = None
-        self.runner: Optional[web.AppRunner] = None
-        self.site: Optional[TCPSite] = None
+        self.webserver: Optional[WebServer] = None
 
         self.connection_close_task: Optional[asyncio.Task] = None
-        self.site_shutdown_task: Optional[asyncio.Task] = None
-        self.app_shut_down_task: Optional[asyncio.Task] = None
         self.received_message_callback: Optional[Callable] = None
         self.api_tasks: Dict[bytes32, asyncio.Task] = {}
         self.execute_tasks: Set[bytes32] = set()
@@ -188,16 +218,6 @@ class ChiaServer:
             ip_network(net, strict=False) for net in config.get("exempt_peer_networks", [])
         ]
 
-    def my_id(self) -> bytes32:
-        """If node has public cert use that one for id, if not use private."""
-        if self.p2p_crt_path is not None:
-            pem_cert = x509.load_pem_x509_certificate(self.p2p_crt_path.read_bytes(), default_backend())
-        else:
-            pem_cert = x509.load_pem_x509_certificate(self._private_cert_path.read_bytes(), default_backend())
-        der_cert_bytes = pem_cert.public_bytes(encoding=serialization.Encoding.DER)
-        der_cert = x509.load_der_x509_certificate(der_cert_bytes, default_backend())
-        return bytes32(der_cert.fingerprint(hashes.SHA256()))
-
     def set_received_message_callback(self, callback: Callable):
         self.received_message_callback = callback
 
@@ -232,7 +252,9 @@ class ChiaServer:
             for peer_ip in to_remove_ban:
                 del self.banned_peers[peer_ip]
 
-    async def start_server(self, on_connect: Callable = None):
+    async def start_server(self, prefer_ipv6: bool, on_connect: Callable = None):
+        if self.webserver is not None:
+            raise RuntimeError("ChiaServer already started")
         if self.incoming_task is None:
             self.incoming_task = asyncio.create_task(self.incoming_api_task())
         if self.gc_task is None:
@@ -241,46 +263,16 @@ class ChiaServer:
         if self._local_type in [NodeType.WALLET, NodeType.HARVESTER, NodeType.TIMELORD]:
             return None
 
-        self.app = web.Application()
         self.on_connect = on_connect
-        routes = [
-            web.get("/ws", self.incoming_connection),
-        ]
-        self.app.add_routes(routes)
-        self.runner = web.AppRunner(self.app, access_log=None, logger=self.log)
-        await self.runner.setup()
-        authenticate = self._local_type not in (NodeType.FULL_NODE, NodeType.INTRODUCER)
-        if authenticate:
-            ssl_context = ssl_context_for_server(
-                self.ca_private_crt_path,
-                self.ca_private_key_path,
-                self._private_cert_path,
-                self._private_key_path,
-                log=self.log,
-            )
-        else:
-            self.p2p_crt_path, self.p2p_key_path = public_ssl_paths(self.root_path, self.config)
-            ssl_context = ssl_context_for_server(
-                self.chia_ca_crt_path, self.chia_ca_key_path, self.p2p_crt_path, self.p2p_key_path, log=self.log
-            )
-
-        # If self._port is set to zero, the socket will bind to a new available port. Therefore, we have to obtain
-        # this port from the socket itself and update self._port.
-        self.site = web.TCPSite(
-            self.runner,
-            host="",  # should listen to both IPv4 and IPv6 on a dual-stack system
-            port=int(self._port),
-            shutdown_timeout=3,
-            ssl_context=ssl_context,
+        self.webserver = await WebServer.create(
+            hostname="",
+            port=uint16(self._port),
+            routes=[web.get("/ws", self.incoming_connection)],
+            ssl_context=self.ssl_context,
+            prefer_ipv6=prefer_ipv6,
+            logger=self.log,
         )
-        await self.site.start()
-        #
-        # On a dual-stack system, we want to get the (first) IPv4 port unless
-        # prefer_ipv6 is set in which case we use the IPv6 port
-        #
-        if self._port == 0:
-            self._port = select_port(self.root_path, self.runner.addresses)
-
+        self._port = int(self.webserver.listen_port)
         self.log.info(f"Started listening on port: {self._port}")
 
     async def incoming_connection(self, request):
@@ -293,7 +285,12 @@ class ChiaServer:
         ws = web.WebSocketResponse(max_msg_size=max_message_size)
         await ws.prepare(request)
         close_event = asyncio.Event()
-        cert_bytes = request.transport._ssl_protocol._extra["ssl_object"].getpeercert(True)
+        ssl_object = request.get_extra_info("ssl_object")
+        if ssl_object is None:
+            reason = f"ssl_object is None for request {request}"
+            self.log.warning(reason)
+            raise web.HTTPInternalServerError(reason=reason)
+        cert_bytes = ssl_object.getpeercert(True)
         der_cert = x509.load_der_x509_certificate(cert_bytes)
         peer_id = bytes32(der_cert.fingerprint(hashes.SHA256()))
         if peer_id == self.node_id:
@@ -369,7 +366,6 @@ class ChiaServer:
             await con.close()
         self.all_connections[connection.peer_node_id] = connection
         if connection.connection_type is not None:
-            self.connection_by_type[connection.connection_type][connection.peer_node_id] = connection
             if on_connect is not None:
                 await on_connect(connection)
         else:
@@ -390,7 +386,6 @@ class ChiaServer:
         self,
         target_node: PeerInfo,
         on_connect: Callable = None,
-        auth: bool = False,
         is_feeler: bool = False,
     ) -> bool:
         """
@@ -404,14 +399,6 @@ class ChiaServer:
             self.log.warning(f"Peer {target_node.host} is still banned, not connecting to it")
             return False
 
-        if auth:
-            ssl_context = ssl_context_for_client(
-                self.ca_private_crt_path, self.ca_private_key_path, self._private_cert_path, self._private_key_path
-            )
-        else:
-            ssl_context = ssl_context_for_client(
-                self.chia_ca_crt_path, self.chia_ca_key_path, self.p2p_crt_path, self.p2p_key_path
-            )
         session = None
         connection: Optional[WSChiaConnection] = None
         try:
@@ -430,20 +417,29 @@ class ChiaServer:
             self.log.debug(f"Connecting: {url}, Peer info: {target_node}")
             try:
                 ws = await session.ws_connect(
-                    url, autoclose=True, autoping=True, heartbeat=60, ssl=ssl_context, max_msg_size=max_message_size
+                    url,
+                    autoclose=True,
+                    autoping=True,
+                    heartbeat=60,
+                    ssl=self.ssl_client_context,
+                    max_msg_size=max_message_size,
                 )
             except ServerDisconnectedError:
                 self.log.debug(f"Server disconnected error connecting to {url}. Perhaps we are banned by the peer.")
                 return False
+            except ClientResponseError as e:
+                self.log.warning(f"Connection failed to {url}. Error: {e}")
+                return False
             except asyncio.TimeoutError:
                 self.log.debug(f"Timeout error connecting to {url}")
                 return False
             if ws is None:
                 return False
 
-            assert ws._response.connection is not None and ws._response.connection.transport is not None
-            transport = ws._response.connection.transport
-            cert_bytes = transport._ssl_protocol._extra["ssl_object"].getpeercert(True)  # type: ignore
+            ssl_object = ws.get_extra_info("ssl_object")
+            if ssl_object is None:
+                raise ValueError(f"ssl_object is None for {ws}")
+            cert_bytes = ssl_object.getpeercert(True)
             der_cert = x509.load_der_x509_certificate(cert_bytes, default_backend())
             peer_id = bytes32(der_cert.fingerprint(hashes.SHA256()))
             if peer_id == self.node_id:
@@ -518,11 +514,8 @@ class ChiaServer:
 
         if connection.peer_node_id in self.all_connections:
             self.all_connections.pop(connection.peer_node_id)
-        if connection.connection_type is not None:
-            if connection.peer_node_id in self.connection_by_type[connection.connection_type]:
-                self.connection_by_type[connection.connection_type].pop(connection.peer_node_id)
-        else:
-            # This means the handshake was enver finished with this peer
+        if connection.connection_type is None:
+            # This means the handshake was never finished with this peer
             self.log.debug(
                 f"Invalid connection type for connection {connection.peer_host},"
                 f" while closing. Handshake never finished."
@@ -544,7 +537,6 @@ class ChiaServer:
             task.cancel()
 
     async def incoming_api_task(self) -> None:
-        self.tasks = set()
         message_types: typing_Counter[str] = Counter()  # Used for debugging information.
         while True:
             payload_inc, connection_inc = await self.incoming_messages.get()
@@ -573,7 +565,8 @@ class ChiaServer:
                         self.log.error(f"Non existing function: {message_type}")
                         raise ProtocolError(Err.INVALID_PROTOCOL_MESSAGE, [message_type])
 
-                    if not hasattr(f, "api_function"):
+                    metadata = get_metadata(function=f)
+                    if not metadata.api_function:
                         self.log.error(f"Peer trying to call non api function {message_type}")
                         raise ProtocolError(Err.INVALID_PROTOCOL_MESSAGE, [message_type])
 
@@ -583,12 +576,12 @@ class ChiaServer:
                             return None
 
                     timeout: Optional[int] = 600
-                    if hasattr(f, "execute_task"):
+                    if metadata.execute_task:
                         # Don't timeout on methods with execute_task decorator, these need to run fully
                         self.execute_tasks.add(task_id)
                         timeout = None
 
-                    if hasattr(f, "peer_required"):
+                    if metadata.peer_required:
                         coroutine = f(full_message.data, connection)
                     else:
                         coroutine = f(full_message.data)
@@ -602,7 +595,7 @@ class ChiaServer:
                         except Exception as e:
                             tb = traceback.format_exc()
                             connection.log.error(f"Exception: {e}, {connection.get_peer_logging()}. {tb}")
-                            raise e
+                            raise
                         return None
 
                     response: Optional[Message] = await asyncio.wait_for(wrapped_coroutine(), timeout=timeout)
@@ -699,15 +692,12 @@ class ChiaServer:
 
     def get_full_node_outgoing_connections(self) -> List[WSChiaConnection]:
         result = []
-        connections = self.get_full_node_connections()
+        connections = self.get_connections(NodeType.FULL_NODE)
         for connection in connections:
             if connection.is_outbound:
                 result.append(connection)
         return result
 
-    def get_full_node_connections(self) -> List[WSChiaConnection]:
-        return list(self.connection_by_type[NodeType.FULL_NODE].values())
-
     def get_connections(self, node_type: Optional[NodeType] = None) -> List[WSChiaConnection]:
         result = []
         for _, connection in self.all_connections.items():
@@ -727,10 +717,8 @@ class ChiaServer:
 
     def close_all(self) -> None:
         self.connection_close_task = asyncio.create_task(self.close_all_connections())
-        if self.runner is not None:
-            self.site_shutdown_task = asyncio.create_task(self.runner.cleanup())
-        if self.app is not None:
-            self.app_shut_down_task = asyncio.create_task(self.app.shutdown())
+        if self.webserver is not None:
+            self.webserver.close()
         for task_id, task in self.api_tasks.items():
             task.cancel()
 
@@ -747,10 +735,9 @@ class ChiaServer:
         await self.shut_down_event.wait()
         if self.connection_close_task is not None:
             await self.connection_close_task
-        if self.app_shut_down_task is not None:
-            await self.app_shut_down_task
-        if self.site_shutdown_task is not None:
-            await self.site_shutdown_task
+        if self.webserver is not None:
+            await self.webserver.await_closed()
+            self.webserver = None
 
     async def get_peer_info(self) -> Optional[PeerInfo]:
         ip = None
@@ -791,7 +778,7 @@ class ChiaServer:
     def accept_inbound_connections(self, node_type: NodeType) -> bool:
         if not self._local_type == NodeType.FULL_NODE:
             return True
-        inbound_count = len([conn for _, conn in self.connection_by_type[node_type].items() if not conn.is_outbound])
+        inbound_count = len([conn for conn in self.get_connections(node_type) if not conn.is_outbound])
         if node_type == NodeType.FULL_NODE:
             return inbound_count < self.config["target_peer_count"] - self.config["target_outbound_peer_count"]
         if node_type == NodeType.WALLET:
