@@ -4,7 +4,7 @@ import time
 import json
 import re
 
-from typing import Dict, Optional, List, Any, Set, Tuple
+from typing import Dict, Optional, List, Any, Set, Tuple, TYPE_CHECKING
 from blspy import AugSchemeMPL, G1Element, G2Element
 from secrets import token_bytes
 
@@ -34,6 +34,7 @@ from chia.wallet.derive_keys import master_sk_to_wallet_sk_unhardened
 from chia.wallet.coin_selection import select_coins
 from chia.wallet.puzzles.p2_delegated_puzzle_or_hidden_puzzle import (
     puzzle_for_pk,
+    puzzle_hash_for_pk,
     DEFAULT_HIDDEN_PUZZLE_HASH,
     calculate_synthetic_secret_key,
 )
@@ -304,7 +305,7 @@ class DIDWallet:
         exclude: Optional[List[Coin]] = None,
         min_coin_amount: Optional[uint64] = None,
         max_coin_amount: Optional[uint64] = None,
-    ) -> Optional[Set[Coin]]:
+    ) -> Set[Coin]:
         """
         Returns a set of coins that can be used for generating a new transaction.
         Note: Must be called under wallet state manager lock
@@ -312,10 +313,10 @@ class DIDWallet:
 
         spendable_amount: uint128 = await self.get_spendable_balance()
 
-        # Only DID Wallet will return none when this happens, so we do it before select_coins would throw an error.
         if amount > spendable_amount:
-            self.log.warning(f"Can't select {amount}, from spendable {spendable_amount} for wallet id {self.id()}")
-            return None
+            error_msg = f"Can't select {amount}, from spendable {spendable_amount} for wallet id {self.id()}"
+            self.log.warning(error_msg)
+            raise ValueError(error_msg)
 
         spendable_coins: List[WalletCoinRecord] = list(
             await self.wallet_state_manager.get_spendable_coins_for_wallet(self.wallet_info.id)
@@ -345,6 +346,30 @@ class DIDWallet:
     async def coin_added(self, coin: Coin, _: uint32, peer: WSChiaConnection):
         """Notification from wallet state manager that wallet has been received."""
 
+        parent = self.get_parent_for_coin(coin)
+        if parent is None:
+            # this is the first time we received it, check it's a DID coin
+            parent_state: CoinState = (
+                await self.wallet_state_manager.wallet_node.get_coin_state([coin.parent_coin_info], peer=peer)
+            )[0]
+            assert parent_state.spent_height is not None
+            puzzle_solution_request = wallet_protocol.RequestPuzzleSolution(
+                coin.parent_coin_info, uint32(parent_state.spent_height)
+            )
+            response = await peer.request_puzzle_solution(puzzle_solution_request)
+            req_puz_sol = response.response
+            assert req_puz_sol.puzzle is not None
+            parent_innerpuz = did_wallet_puzzles.get_innerpuzzle_from_puzzle(req_puz_sol.puzzle.to_program())
+            if parent_innerpuz:
+                parent_info = LineageProof(
+                    parent_state.coin.parent_coin_info,
+                    parent_innerpuz.get_tree_hash(),
+                    uint64(parent_state.coin.amount),
+                )
+                await self.add_parent(coin.parent_coin_info, parent_info)
+            else:
+                self.log.warning("Parent coin is not a DID, skipping: %s -> %s", coin.name(), coin)
+                return
         self.log.info(f"DID wallet has been notified that coin was added: {coin.name()}:{coin}")
         inner_puzzle = await self.inner_puzzle_for_did_puzzle(coin.puzzle_hash)
         if self.did_info.temp_coin is not None:
@@ -370,26 +395,6 @@ class DIDWallet:
         )
 
         await self.add_parent(coin.name(), future_parent)
-        parent = self.get_parent_for_coin(coin)
-        if parent is None:
-            parent_state: CoinState = (
-                await self.wallet_state_manager.wallet_node.get_coin_state([coin.parent_coin_info], peer=peer)
-            )[0]
-            assert parent_state.spent_height is not None
-            puzzle_solution_request = wallet_protocol.RequestPuzzleSolution(
-                coin.parent_coin_info, parent_state.spent_height
-            )
-            response = await peer.request_puzzle_solution(puzzle_solution_request)
-            req_puz_sol = response.response
-            assert req_puz_sol.puzzle is not None
-            parent_innerpuz = did_wallet_puzzles.get_innerpuzzle_from_puzzle(req_puz_sol.puzzle.to_program())
-            assert parent_innerpuz is not None
-            parent_info = LineageProof(
-                parent_state.coin.parent_coin_info,
-                parent_innerpuz.get_tree_hash(),
-                uint64(parent_state.coin.amount),
-            )
-            await self.add_parent(coin.parent_coin_info, parent_info)
 
     def create_backup(self) -> str:
         """
@@ -508,6 +513,20 @@ class DIDWallet:
             innerpuz = Program.to((8, 0))
             return did_wallet_puzzles.create_fullpuz(innerpuz, bytes32([0] * 32))
 
+    def puzzle_hash_for_pk(self, pubkey: G1Element) -> bytes32:
+        if self.did_info.origin_coin is None:
+            # TODO: this seem dumb. Why bother with this case? Is it ever used?
+            return puzzle_for_pk(pubkey).get_tree_hash()
+        origin_coin_name = self.did_info.origin_coin.name()
+        innerpuz_hash = did_wallet_puzzles.get_inner_puzhash_by_p2(
+            puzzle_hash_for_pk(pubkey),
+            self.did_info.backup_ids,
+            self.did_info.num_of_backup_ids_needed,
+            origin_coin_name,
+            did_wallet_puzzles.metadata_to_program(json.loads(self.did_info.metadata)),
+        )
+        return did_wallet_puzzles.create_fullpuz_hash(innerpuz_hash, origin_coin_name)
+
     async def get_new_puzzle(self) -> Program:
         return self.puzzle_for_pk(
             (await self.wallet_state_manager.get_unused_derivation_record(self.wallet_info.id)).pubkey
@@ -578,7 +597,7 @@ class DIDWallet:
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=new_puzhash,
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             amount=uint64(coin.amount),
             fee_amount=uint64(0),
             confirmed=False,
@@ -672,7 +691,7 @@ class DIDWallet:
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=new_puzhash,
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             amount=uint64(coin.amount),
             fee_amount=fee,
             confirmed=False,
@@ -775,7 +794,7 @@ class DIDWallet:
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=puzhash,
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             amount=uint64(coin.amount),
             fee_amount=uint64(0),
             confirmed=False,
@@ -849,7 +868,7 @@ class DIDWallet:
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=coin.puzzle_hash,
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             amount=uint64(coin.amount),
             fee_amount=uint64(0),
             confirmed=False,
@@ -977,7 +996,7 @@ class DIDWallet:
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=puzhash,
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             amount=uint64(coin.amount),
             fee_amount=uint64(0),
             confirmed=False,
@@ -1080,6 +1099,22 @@ class DIDWallet:
 
         return parent_info
 
+    async def sign_message(self, message: str) -> Tuple[G1Element, G2Element]:
+        if self.did_info.current_inner is None:
+            raise ValueError("Missing DID inner puzzle.")
+        puzzle_args = did_wallet_puzzles.uncurry_innerpuz(self.did_info.current_inner)
+        if puzzle_args is not None:
+
+            p2_puzzle, _, _, _, _ = puzzle_args
+            puzzle_hash = p2_puzzle.get_tree_hash()
+            pubkey, private = await self.wallet_state_manager.get_keys(puzzle_hash)
+            synthetic_secret_key = calculate_synthetic_secret_key(private, DEFAULT_HIDDEN_PUZZLE_HASH)
+            synthetic_pk = synthetic_secret_key.get_g1()
+            puzzle: Program = Program.to(("Chia Signed Message", message))
+            return synthetic_pk, AugSchemeMPL.sign(synthetic_secret_key, puzzle.get_tree_hash())
+        else:
+            raise ValueError("Invalid inner DID puzzle.")
+
     async def sign(self, spend_bundle: SpendBundle) -> SpendBundle:
         sigs: List[G2Element] = []
         for spend in spend_bundle.coin_spends:
@@ -1120,7 +1155,7 @@ class DIDWallet:
             return None
 
         origin = coins.copy().pop()
-        genesis_launcher_puz = did_wallet_puzzles.SINGLETON_LAUNCHER
+        genesis_launcher_puz = did_wallet_puzzles.LAUNCHER_PUZZLE
         launcher_coin = Coin(origin.name(), genesis_launcher_puz.get_tree_hash(), amount)
 
         did_inner: Program = await self.get_new_did_innerpuz(launcher_coin.name())
@@ -1175,15 +1210,12 @@ class DIDWallet:
         full_spend = SpendBundle.aggregate([tx_record.spend_bundle, eve_spend, launcher_sb])
         assert self.did_info.origin_coin is not None
         assert self.did_info.current_inner is not None
-        did_puzzle_hash = did_wallet_puzzles.create_fullpuz(
-            self.did_info.current_inner, self.did_info.origin_coin.name()
-        ).get_tree_hash()
 
         did_record = TransactionRecord(
             confirmed_at_height=uint32(0),
             created_at_time=uint64(int(time.time())),
-            to_puzzle_hash=did_puzzle_hash,
             amount=uint64(amount),
+            to_puzzle_hash=await self.standard_wallet.get_puzzle_hash(False),
             fee_amount=fee,
             confirmed=False,
             sent=uint32(10),
@@ -1239,7 +1271,7 @@ class DIDWallet:
         )
         return spendable_am
 
-    async def get_max_send_amount(self, records=None):
+    async def get_max_send_amount(self, records: Optional[Set[WalletCoinRecord]] = None):
         max_send_amount = await self.get_confirmed_balance()
 
         return max_send_amount
@@ -1363,3 +1395,12 @@ class DIDWallet:
             metadata,
         )
         return did_info
+
+    def require_derivation_paths(self) -> bool:
+        return True
+
+
+if TYPE_CHECKING:
+    from chia.wallet.wallet_protocol import WalletProtocol
+
+    _dummy: WalletProtocol = DIDWallet()
