@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 import asyncio
 import time
 from typing import Any, Awaitable, Callable, Dict, List
 
 import pytest
+from blspy import AugSchemeMPL, G1Element, G2Element
 from clvm_tools.binutils import disassemble
 
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
@@ -34,8 +37,8 @@ async def tx_in_pool(mempool: MempoolManager, tx_id: bytes32) -> bool:
     return True
 
 
-async def get_nft_number(wallet: NFTWallet) -> int:
-    return len(await wallet.load_current_nft())
+async def get_nft_count(wallet: NFTWallet) -> int:
+    return await wallet.get_nft_count()
 
 
 async def get_wallet_number(manager: WalletStateManager) -> int:
@@ -44,21 +47,28 @@ async def get_wallet_number(manager: WalletStateManager) -> int:
 
 async def wait_rpc_state_condition(
     timeout: int,
-    coroutine: Callable[[Dict[str, Any]], Awaitable[Dict]],
+    async_function: Callable[[Dict[str, Any]], Awaitable[Dict]],
     params: List[Dict],
     condition_func: Callable[[Dict[str, Any]], bool],
 ) -> Dict:
+    __tracebackhide__ = True
+
     start = time.monotonic()
-    resp = None
-    while time.monotonic() - start < timeout:
-        resp = await coroutine(*params)
+
+    while True:
+        resp = await async_function(*params)
         assert isinstance(resp, dict)
         if condition_func(resp):
             return resp
+
+        now = time.monotonic()
+        elapsed = now - start
+        if elapsed >= timeout:
+            raise asyncio.TimeoutError(
+                f"timed out while waiting for {async_function.__name__}(): {elapsed} >= {timeout}",
+            )
+
         await asyncio.sleep(0.5)
-    # timed out
-    assert time.monotonic() - start < timeout, resp
-    return {}
 
 
 async def make_new_block_with(resp: Dict, full_node_api: FullNodeSimulator, ph: bytes32) -> SpendBundle:
@@ -135,8 +145,8 @@ async def test_nft_wallet_creation_automatically(two_wallet_nodes: Any, trusted:
     for i in range(1, num_blocks):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
 
-    await time_out_assert(30, len, 1, nft_wallet_0.my_nft_coins)
-    coins = nft_wallet_0.my_nft_coins
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_0)
+    coins = await nft_wallet_0.get_current_nfts()
     assert len(coins) == 1, "nft not generated"
 
     txs = await nft_wallet_0.generate_signed_transaction([uint64(coins[0].coin.amount)], [ph1], coins={coins[0].coin})
@@ -157,12 +167,12 @@ async def test_nft_wallet_creation_automatically(two_wallet_nodes: Any, trusted:
     nft_wallets = await wallet_node_1.wallet_state_manager.get_all_wallet_info_entries(WalletType.NFT)
     assert len(nft_wallets) == 1
     nft_wallet_1: NFTWallet = wallet_node_1.wallet_state_manager.wallets[nft_wallets[0].id]
+    await time_out_assert(30, get_nft_count, 0, nft_wallet_0)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_1)
 
-    await time_out_assert(30, len, 0, nft_wallet_0.my_nft_coins)
-    await time_out_assert(30, len, 1, nft_wallet_1.my_nft_coins)
-    coins = nft_wallet_0.my_nft_coins
+    coins = await nft_wallet_0.get_current_nfts()
     assert len(coins) == 0
-    coins = nft_wallet_1.my_nft_coins
+    coins = await nft_wallet_1.get_current_nfts()
     assert len(coins) == 1
 
 
@@ -180,7 +190,6 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
     wallet_node_1, server_1 = wallets[1]
     wallet_0 = wallet_node_0.wallet_state_manager.main_wallet
     wallet_1 = wallet_node_1.wallet_state_manager.main_wallet
-
     ph = await wallet_0.get_new_puzzlehash()
     ph1 = await wallet_1.get_new_puzzlehash()
 
@@ -235,7 +244,7 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
     for i in range(1, num_blocks):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-    await time_out_assert(10, len, 1, nft_wallet_0.my_nft_coins)
+    await time_out_assert(10, get_nft_count, 1, nft_wallet_0)
     await time_out_assert(10, wallet_0.get_unconfirmed_balance, 4000000000000 - 1)
     await time_out_assert(10, wallet_0.get_confirmed_balance, 4000000000000 - 1)
     # Test Reorg mint
@@ -243,7 +252,7 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
     if height is None:
         assert False
     await full_node_api.reorg_from_index_to_new_index(ReorgProtocol(uint32(height - 1), uint32(height + 1), ph1, None))
-    await time_out_assert(30, get_nft_number, 0, nft_wallet_0)
+    await time_out_assert(30, get_nft_count, 0, nft_wallet_0)
     await time_out_assert(30, get_wallet_number, 1, wallet_node_0.wallet_state_manager)
 
     nft_wallet_0 = await NFTWallet.create_new_nft_wallet(
@@ -269,8 +278,8 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
     await time_out_assert(30, wallet_node_0.wallet_state_manager.lock.locked, False)
     for i in range(1, num_blocks * 2):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
-    await time_out_assert(30, get_nft_number, 2, nft_wallet_0)
-    coins = nft_wallet_0.my_nft_coins
+    await time_out_assert(30, get_nft_count, 2, nft_wallet_0)
+    coins = await nft_wallet_0.get_current_nfts()
     assert len(coins) == 2, "nft not generated"
 
     await time_out_assert(30, wallet_0.get_pending_change_balance, 0)
@@ -288,10 +297,10 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
 
     for i in range(1, num_blocks * 2):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
-    await time_out_assert(30, get_nft_number, 1, nft_wallet_0)
-    await time_out_assert(30, get_nft_number, 1, nft_wallet_1)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_0)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_1)
 
-    coins = nft_wallet_1.my_nft_coins
+    coins = await nft_wallet_1.get_current_nfts()
     assert len(coins) == 1
 
     await time_out_assert(30, wallet_1.get_pending_change_balance, 0)
@@ -310,16 +319,16 @@ async def test_nft_wallet_creation_and_transfer(two_wallet_nodes: Any, trusted:
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
 
     await time_out_assert(30, wallet_node_0.wallet_state_manager.lock.locked, False)
-    await time_out_assert(30, get_nft_number, 2, nft_wallet_0)
-    await time_out_assert(30, get_nft_number, 0, nft_wallet_1)
+    await time_out_assert(30, get_nft_count, 2, nft_wallet_0)
+    await time_out_assert(30, get_nft_count, 0, nft_wallet_1)
 
     # Test Reorg
     height = full_node_api.full_node.blockchain.get_peak_height()
     if height is None:
         assert False
     await full_node_api.reorg_from_index_to_new_index(ReorgProtocol(uint32(height - 1), uint32(height + 2), ph1, None))
-    await time_out_assert(30, get_nft_number, 1, nft_wallet_0)
-    await time_out_assert(30, get_nft_number, 1, nft_wallet_1)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_0)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_1)
 
 
 @pytest.mark.parametrize(
@@ -394,6 +403,10 @@ async def test_nft_wallet_rpc_creation_and_list(two_wallet_nodes: Any, trusted:
             "artist_address": ph,
             "hash": "0xD4584AD463139FA8C0D9F68F4B59F184",
             "uris": ["https://chialisp.com/img/logo.svg"],
+            "meta_uris": [
+                "https://bafybeigzcazxeu7epmm4vtkuadrvysv74lbzzbl2evphtae6k57yhgynp4.ipfs.nftstorage.link/6590.json"
+            ],
+            "meta_hash": "0x6a9cb99b7b9a987309e8dd4fd14a7ca2423858585da68cc9ec689669dd6dd6ab",
         }
     )
     assert isinstance(tr2, dict)
@@ -403,7 +416,10 @@ async def test_nft_wallet_rpc_creation_and_list(two_wallet_nodes: Any, trusted:
     for i in range(1, num_blocks):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
     coins_response = await wait_rpc_state_condition(
-        5, api_0.nft_get_nfts, [{"wallet_id": nft_wallet_0_id}], lambda x: x["success"] and len(x["nft_list"]) == 2
+        5,
+        api_0.nft_get_nfts,
+        [{"wallet_id": nft_wallet_0_id}],
+        lambda x: x["success"] and len(x["nft_list"]) == 2,
     )
     coins = coins_response["nft_list"]
     uris = []
@@ -415,6 +431,16 @@ async def test_nft_wallet_rpc_creation_and_list(two_wallet_nodes: Any, trusted:
     assert "https://chialisp.com/img/logo.svg" in uris
     assert bytes32.fromhex(coins[1].to_json_dict()["nft_coin_id"][2:]) in [x.name() for x in sb.additions()]
 
+    coins_response = await wait_rpc_state_condition(
+        5,
+        api_0.nft_get_nfts,
+        [{"wallet_id": nft_wallet_0_id, "start_index": 1, "num": 1}],
+        lambda x: x["success"] and len(x["nft_list"]) == 1,
+    )
+    coins = coins_response["nft_list"]
+    assert len(coins) == 1
+    assert coins[0].data_hash.hex() == "0xD4584AD463139FA8C0D9F68F4B59F184"[2:].lower()
+
 
 @pytest.mark.parametrize(
     "trusted",
@@ -928,6 +954,7 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
             "hash": "0xD4584AD463139FA8C0D9F68F4B59F185",
             "uris": ["https://www.chia.net/img/branding/chia-logo.svg"],
             "fee": fee,
+            "did_id": hmr_did_id,
         }
     )
     await make_new_block_with(resp, full_node_api, ph1)
@@ -962,15 +989,13 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     for i in range(1, num_blocks):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph1))
 
-    await wait_rpc_state_condition(
-        5, api_0.nft_get_nfts, [dict(wallet_id=nft_wallet_0_id)], lambda x: not x["nft_list"]
-    )
+    await time_out_assert(5, len, 2, wallet_0.wallet_state_manager.wallets)
     await time_out_assert(30, wallet_0.get_unconfirmed_balance, 5999999999798)
     await time_out_assert(30, wallet_0.get_confirmed_balance, 5999999999798)
     # wait for all wallets to be created
     await time_out_assert(30, len, 3, wallet_1.wallet_state_manager.wallets)
     did_wallet_1 = wallet_1.wallet_state_manager.wallets[3]
-    assert len(wallet_node_0.wallet_state_manager.wallets[nft_wallet_0_id].my_nft_coins) == 0
+    assert nft_wallet_0_id not in wallet_node_0.wallet_state_manager.wallets.keys()
     # Check if the NFT owner DID is reset
     resp = await api_1.nft_get_by_did({})
     assert resp.get("success")
@@ -980,6 +1005,7 @@ async def test_nft_transfer_nft_with_did(two_wallet_nodes: Any, trusted: Any) ->
     )
     assert len(coins_response["nft_list"]) == 1
     assert coins_response["nft_list"][0].owner_did is None
+    assert coins_response["nft_list"][0].minter_did.hex() == hex_did_id
     nft_coin_id = coins_response["nft_list"][0].nft_coin_id
 
     await time_out_assert(30, did_wallet_1.get_spendable_balance, 1)
@@ -1079,6 +1105,7 @@ async def test_update_metadata_for_nft_did(two_wallet_nodes: Any, trusted: Any)
             "hash": "0xD4584AD463139FA8C0D9F68F4B59F185",
             "uris": ["https://www.chia.net/img/branding/chia-logo.svg"],
             "mu": ["https://www.chia.net/img/branding/chia-logo.svg"],
+            "did": hex_did_id,
         }
     )
     assert resp.get("success")
@@ -1098,6 +1125,7 @@ async def test_update_metadata_for_nft_did(two_wallet_nodes: Any, trusted: Any)
     )
     coins = coins_response["nft_list"]
     assert len(coins) == 1
+    assert coins[0].minter_did.hex() == hex_did_id
     nft_coin_id = coins[0].nft_coin_id
 
     # add another URI
@@ -1114,6 +1142,7 @@ async def test_update_metadata_for_nft_did(two_wallet_nodes: Any, trusted: Any)
     assert tr1.get("success")
     coins_response = await api_0.nft_get_nfts(dict(wallet_id=nft_wallet_0_id))
     assert coins_response["nft_list"][0].pending_transaction
+
     sb = tr1["spend_bundle"]
     await time_out_assert_not_none(30, full_node_api.full_node.mempool_manager.get_spendbundle, sb.name())
     for i in range(1, num_blocks):
@@ -1129,6 +1158,7 @@ async def test_update_metadata_for_nft_did(two_wallet_nodes: Any, trusted: Any)
     )
 
     coin = coins_response["nft_info"].to_json_dict()
+    assert coin["minter_did"][2:] == hex_did_id
     assert coin["mint_height"] > 0
     uris = coin["data_uris"]
     assert len(uris) == 1
@@ -1248,7 +1278,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
         [dict(wallet_id=nft_wallet_1_id)],
         lambda x: len(x["nft_list"]) > 0 and x["nft_list"][0].owner_did,
     )
-    assert len(wallet_node_0.wallet_state_manager.wallets[nft_wallet_0_id].my_nft_coins) == 0
+    assert len(await wallet_node_0.wallet_state_manager.wallets[nft_wallet_0_id].get_current_nfts()) == 0
 
     coins = resp["nft_list"]
     assert len(coins) == 1
@@ -1273,7 +1303,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
 
     nft_wallet_2_id = coins_response.get("wallet_id")
     assert nft_wallet_2_id
-    assert len(wallet_node_0.wallet_state_manager.wallets[nft_wallet_1_id].my_nft_coins) == 0
+    await time_out_assert(30, wallet_node_0.wallet_state_manager.wallets.get, 0, nft_wallet_1_id, 0)
 
     # Check NFT DID
     resp = await wait_rpc_state_condition(
@@ -1298,7 +1328,7 @@ async def test_nft_set_did(two_wallet_nodes: Any, trusted: Any) -> None:
     coins = resp["nft_list"]
     assert len(coins) == 1
     assert coins[0].owner_did is None
-    assert len(wallet_node_0.wallet_state_manager.wallets[nft_wallet_2_id].my_nft_coins) == 0
+    assert nft_wallet_2_id not in wallet_node_0.wallet_state_manager.wallets.keys()
     nft_coin_id = coins[0].nft_coin_id
     resp = await api_0.nft_get_info(dict(coin_id=nft_coin_id.hex(), latest=True))
     assert resp["success"]
@@ -1388,3 +1418,87 @@ async def test_set_nft_status(two_wallet_nodes: Any, trusted: Any) -> None:
     assert coins_response.get("success")
     coins = coins_response["nft_list"]
     assert coins[0].pending_transaction
+
+
+@pytest.mark.parametrize(
+    "trusted",
+    [True, False],
+)
+@pytest.mark.asyncio
+async def test_nft_sign_message(two_wallet_nodes: Any, trusted: Any) -> None:
+    num_blocks = 5
+    full_nodes, wallets, _ = two_wallet_nodes
+    full_node_api: FullNodeSimulator = full_nodes[0]
+    full_node_server = full_node_api.server
+    wallet_node_0, server_0 = wallets[0]
+    wallet_node_1, server_1 = wallets[1]
+    wallet_0 = wallet_node_0.wallet_state_manager.main_wallet
+    api_0 = WalletRpcApi(wallet_node_0)
+    ph = await wallet_0.get_new_puzzlehash()
+
+    if trusted:
+        wallet_node_0.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+        wallet_node_1.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+    else:
+        wallet_node_0.config["trusted_peers"] = {}
+        wallet_node_1.config["trusted_peers"] = {}
+
+    await server_0.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+    await server_1.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+
+    for _ in range(1, num_blocks):
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
+
+    funds = sum(
+        [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks - 1)]
+    )
+
+    await time_out_assert(30, wallet_0.get_unconfirmed_balance, funds)
+    await time_out_assert(30, wallet_0.get_confirmed_balance, funds)
+    await time_out_assert(30, wallet_is_synced, True, wallet_node_0, full_node_api)
+    res = await api_0.create_new_wallet(dict(wallet_type="nft_wallet", name="NFT WALLET 1"))
+    assert isinstance(res, dict)
+    assert res.get("success")
+    nft_wallet_0_id = res["wallet_id"]
+
+    # Create a NFT without DID
+    resp = await api_0.nft_mint_nft(
+        {
+            "wallet_id": nft_wallet_0_id,
+            "hash": "0xD4584AD463139FA8C0D9F68F4B59F185",
+            "uris": ["https://www.chia.net/img/branding/chia-logo.svg"],
+            "mu": ["https://www.chia.net/img/branding/chia-logo.svg"],
+        }
+    )
+    assert resp.get("success")
+    sb = resp["spend_bundle"]
+
+    # ensure hints are generated
+    assert compute_memos(sb)
+    await time_out_assert_not_none(30, full_node_api.full_node.mempool_manager.get_spendbundle, sb.name())
+    await make_new_block_with(resp, full_node_api, ph)
+
+    # Check DID NFT
+    coins_response = await wait_rpc_state_condition(
+        30, api_0.nft_get_nfts, [dict(wallet_id=nft_wallet_0_id)], lambda x: len(x["nft_list"]) > 0
+    )
+    assert coins_response["nft_list"], isinstance(coins_response, dict)
+    assert coins_response.get("success")
+    coins = coins_response["nft_list"]
+    assert len(coins) == 1
+    assert coins[0].owner_did is None
+    assert not coins[0].pending_transaction
+    message = "Hello World"
+    response = await api_0.sign_message_by_id(
+        {"id": encode_puzzle_hash(coins[0].launcher_id, AddressType.NFT.value), "message": message}
+    )
+    puzzle: Program = Program.to(("Chia Signed Message", message))
+    assert AugSchemeMPL.verify(
+        G1Element.from_bytes(bytes.fromhex(response["pubkey"])),
+        puzzle.get_tree_hash(),
+        G2Element.from_bytes(bytes.fromhex(response["signature"])),
+    )
